--[[

Lizard Demo Room Debug
Robert Hart (rnhart.net)

How to use:

1. Open lizard_demo.nes in FCEUX.
2. Drag room-debug.lua into the FCEUX window.

Number in center of screen: current room number
Numbers on edges of screen: room linked in that direction
Number in upper-right.....: room linked on the other side (through door)

direction + SELECT: go to room linked in direction
A + SELECT: go to room on other side (through door)
SELECT: increment room number

]]

LEFT  = 10
RIGHT = 240
MIDX  = 125

TOP    = 10
BOTTOM = 220
MIDY   = 115

DOORX = MIDX + 50
DOORY = MIDY - 50

    --  top   bottom  left  right door
dir  = {0xD,  0xE,    0xB,  0xC,   1    }
xpos = {MIDX, MIDX,   LEFT, RIGHT, DOORX}
ypos = {TOP,  BOTTOM, MIDY, MIDY,  DOORY}
roomlink = {0,     0,    0,    0,      0}


FIRST_ROOM_WRITE = 0x8F0C
ROOM_NUMBER = 0x2D

room = 0

function PrintRoomLinks()
  current_room = memory.readbyte(ROOM_NUMBER)

  gui.text(MIDX, MIDY, current_room)

  for i = 1, #dir do
    Y = dir[i]
    X = memory.readbyte(0x430 + Y)
    new_room = memory.readbyte(0x45 + X)
    gui.text(xpos[i], ypos[i], new_room)
    roomlink[i] = new_room
  end

end

function CatchFirstRoomWrite()
  memory.registerexec(FIRST_ROOM_WRITE, nil)
  memory.setregister("a", room)
  emu.registerafter(CheckRelease)
end

function NextRoom()
  buttons = joypad.get(1)

  if     (buttons["up"   ] == true) then room = roomlink[1]
  elseif (buttons["down" ] == true) then room = roomlink[2]
  elseif (buttons["left" ] == true) then room = roomlink[3]
  elseif (buttons["right"] == true) then room = roomlink[4]
  elseif (buttons["A"    ] == true) then room = roomlink[5]
  else                                   room = room + 1
  end

  memory.registerexec(FIRST_ROOM_WRITE, CatchFirstRoomWrite)
  emu.softreset()
end

function CheckPress()
  buttons = joypad.get(1)
  if(buttons["select"] == true) then
    emu.registerafter(nil)
    NextRoom()
  end
  PrintRoomLinks()
end

function CheckRelease()
  buttons = joypad.get(1)
  if(buttons["select"] == false) then
    emu.registerafter(CheckPress)
  end
  PrintRoomLinks()
end

emu.registerafter(CheckRelease)

emu.print("number in center of screen: current room number")
emu.print("numbers on edges of screen: room linked in that direction")
emu.print("number in upper-right: room linked on the other side (through door)")
emu.print("direction + SELECT: go to room linked in direction")
emu.print("A + SELECT: go to room on other side (through door)")
emu.print("SELECT: increment room number")
